﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents a description of a complex linear equation system.
    /// </summary>
    [Serializable]
    public class ComplexLinearEquationSystem
    {
        /// <summary>
        /// The complex coefficient matrix of the linear equation system.
        /// </summary>
        private ComplexMatrix coefficientMatrix;

        /// <summary>
        /// The complex result vector of the linear equation system.
        /// </summary>
        private GeneralComplexVector resultVector;

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexLinearEquationSystem"/> class.
        /// </summary>
        /// <param name="coefficientMatrix">The coefficient matrix of the complex linear equation system.</param>
        /// <param name="resultVector">The result vector of the complex linear equation system.</param>
        public ComplexLinearEquationSystem(ComplexMatrix coefficientMatrix, GeneralComplexVector resultVector)
        {
            if (coefficientMatrix.Rows != resultVector.Count)
            {
                throw new NotConfortableException(
                    "The number of rows of the coefficient matrix and the result vector have to be even.");
            }

            this.coefficientMatrix = coefficientMatrix;
            this.resultVector = resultVector;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexLinearEquationSystem"/> class.
        /// </summary>
        /// <param name="equationSystem">The equation system to clone.</param>
        public ComplexLinearEquationSystem(ComplexLinearEquationSystem equationSystem)
        {
            this.coefficientMatrix = equationSystem.CoefficientMatrix.Copy();
            this.resultVector = equationSystem.ResultVector.Copy();
        }

        /// <summary>
        /// The complex coefficient matrix of the linear equation system.
        /// </summary>
        public ComplexMatrix CoefficientMatrix
        {
            get { return coefficientMatrix; }
            set { coefficientMatrix = value; }
        }

        /// <summary>
        /// The complex result vector of the linear equation system.
        /// </summary>
        public GeneralComplexVector ResultVector
        {
            get { return resultVector; }
            set { resultVector = value; }
        }
    }
}