﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents a complex nonlinear system of equation.
    /// </summary>
    [Serializable]
    public class ComplexNonLinearEquationSystem
    {
        /// <summary>
        /// The functions, which define the complex nonlinear real system of equation.
        /// </summary>
        private HardMultivariateComplexFunction[] equationSystem;

        /// <param name="equationSystem">The functions, which define the complex nonlinear system of equation.</param>
        public ComplexNonLinearEquationSystem(HardMultivariateComplexFunction[] equationSystem)
        {
            if (equationSystem == (HardMultivariateComplexFunction[]) null)
            {
                throw new ArgumentNullException("equationSystem");
            }

            this.equationSystem = equationSystem;
        }

        /// <summary>
        /// Gets or sets the functions, which define the complex nonlinear system of equation.
        /// </summary>
        /// <value>The functions, which define the complex nonlinear system of equation.</value>
        public HardMultivariateComplexFunction[] EquationSystem
        {
            get { return equationSystem; }
            set { equationSystem = value; }
        }

        /// <summary>
        /// Solves all equations in complex nonlinear system of equation at the specified vector.
        /// </summary>
        /// <param name="vector">The specified vector.</param>
        /// <returns>The results of the complex equations in the system.</returns>
        public GeneralComplexVector SolveAt(GeneralComplexVector vector)
        {
            GeneralComplexVector result = new GeneralComplexVector(this.equationSystem.Length);

            for (int i = 0; i < this.equationSystem.Length; i++)
            {
                result[i] = this.equationSystem[i].SolveAt(vector);
            }

            return result;
        }
    }
}