﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Generator
{
    /// <summary>
    /// This class provides a function generator for the real Gamma function.
    /// </summary>
    [Serializable]
    public class GammaFunctionGenerator : AbstractOneParameterFunctionGenerator
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="GammaFunctionGenerator"/> class.
        /// </summary>
        /// <param name="interval">The interval of the function generator.</param>
        public GammaFunctionGenerator(double interval)
            : base(interval)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GammaFunctionGenerator"/> class.
        /// </summary>
        /// <param name="interval">The interval of the function generator.</param>
        /// <param name="startvalue">The start value of the function generator.</param>
        public GammaFunctionGenerator(double interval, double startvalue)
            : base(interval, startvalue)
        {
        }

        /// <summary>
        /// Calculates the next value by using the Gamma function.
        /// </summary>
        /// <returns>The calculated value of the generator.</returns>
        public override double NextValue()
        {
            double result = SpecialFunctions.GammaFunction(this.currentValue);

            this.currentValue += this.Interval;

            return result;
        }
    }
}