﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Generator
{
    /// <summary>
    /// This class provides a function generator for the real Zeta function.
    /// </summary>
    [Serializable]
    public class ZetaFunctionGenerator : AbstractOneParameterIntegrationGenerator
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ZetaFunctionGenerator"/> class.
        /// </summary>
        /// <param name="interval">The interval of the function generator.</param>
        public ZetaFunctionGenerator(double interval)
            : base(interval)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ZetaFunctionGenerator"/> class.
        /// </summary>
        /// <param name="interval">The interval of the function generator.</param>
        /// <param name="startvalue">The start value of the function generator.</param>
        public ZetaFunctionGenerator(double interval, double startvalue)
            : base(interval, startvalue)
        {
        }

        /// <summary>
        /// Calculates the next value by using the Zeta function.
        /// </summary>
        /// <returns>The calculated value of the generator.</returns>
        public override double NextValue()
        {
            double result = SpecialFunctions.ZetaFunction(this.currentValue);

            this.currentValue += this.Interval;

            return result;
        }

        /// <summary>
        /// Calculates the next value by using the Zeta function.
        /// </summary>
        /// <param name="limit">The limit of integration.</param>
        /// <returns>The calculated value of the generator.</returns>
        public override double NextValue(int limit)
        {
            double result = SpecialFunctions.ZetaFunction(this.currentValue, limit);

            this.currentValue += this.Interval;

            return result;
        }
    }
}