﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// Defines a function delegate to bind the method, which represents the bivariate complex 
    /// function with two complex arguments.
    /// </summary>
    public delegate ComplexNumber HardBivariateComplexFunctionPointer(ComplexNumber a, ComplexNumber b);

    /// <summary>
    /// This class represents a bivariate function, which can be implemented by source code.
    /// </summary>
    [Serializable]
    public class HardBivariateComplexFunction : IHardBivariateComplexFunction
    {
        /// <summary>
        /// The pointer to the method, which represents the function.
        /// </summary>
        private HardBivariateComplexFunctionPointer hardComplexFunctionPointer;

        /// <summary>
        /// Initializes a new instance of the <see cref="HardBivariateComplexFunction"/> class.
        /// </summary>
        /// <param name="hardComplexFunctionPointer">The pointer to the method, which represents the function.</param>
        public HardBivariateComplexFunction(HardBivariateComplexFunctionPointer hardComplexFunctionPointer)
        {
            this.hardComplexFunctionPointer = hardComplexFunctionPointer;
        }

        /// <summary>
        /// Gets or sets the pointer to the method, which represents the function.
        /// </summary>
        /// <value>The pointer to the method, which represents the function.</value>
        public HardBivariateComplexFunctionPointer HardComplexFunctionPointer
        {
            get { return hardComplexFunctionPointer; }
            set { hardComplexFunctionPointer = value; }
        }

        /// <summary>
        /// Calculates the function at the specified values a and b.
        /// </summary>
        /// <param name="a">The first parameter of the function.</param>
        /// <param name="b">The second parameter of the function.</param>
        /// <returns>
        /// The result of the function at the specified values a and b.
        /// </returns>
        public ComplexNumber SolveAt(ComplexNumber a, ComplexNumber b)
        {
            return this.hardComplexFunctionPointer(a, b);
        }
    }
}