﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// Defines a function delegate to bind the method, which represents the multivariate complex 
    /// function.
    /// </summary>
    public delegate ComplexNumber HardMultivariateComplexFunctionPointer(GeneralComplexVector vector);

    /// <summary>
    /// This class represents a multivariate function, which can be implemented by source code.
    /// </summary>
    [Serializable]
    public class HardMultivariateComplexFunction : IHardMultivariateComplexFunction
    {
        /// <summary>
        /// The pointer to the method, which represents the function.
        /// </summary>
        private HardMultivariateComplexFunctionPointer hardComplexFunctionPointer;

        /// <summary>
        /// Initializes a new instance of the <see cref="HardMultivariateComplexFunction"/> class.
        /// </summary>
        /// <param name="hardComplexFunctionPointer">The pointer to the method, which represents the function.</param>
        public HardMultivariateComplexFunction(HardMultivariateComplexFunctionPointer hardComplexFunctionPointer)
        {
            this.hardComplexFunctionPointer = hardComplexFunctionPointer;
        }

        /// <summary>
        /// Gets or sets the pointer to the method, which represents the function.
        /// </summary>
        /// <value>The pointer to the method, which represents the function.</value>
        public HardMultivariateComplexFunctionPointer HardComplexFunctionPointer
        {
            get { return hardComplexFunctionPointer; }
            set { hardComplexFunctionPointer = value; }
        }

        /// <summary>
        /// Calculates the function at the specified vector.
        /// </summary>
        /// <param name="vector">The input values for the function in form of a vector.</param>
        /// <returns>
        /// The result of the function at the specified vector.
        /// </returns>
        public ComplexNumber SolveAt(GeneralComplexVector vector)
        {
            return this.hardComplexFunctionPointer(vector);
        }
    }
}