﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// Defines a function delegate to bind the method, which represents the multivariate real 
    /// function.
    /// </summary>
    public delegate double HardMultivariateRealFunctionPointer(GeneralVector vector);

    /// <summary>
    /// This class represents a multivariate function, which can be implemented by source code.
    /// </summary>
    [Serializable]
    public class HardMultivariateRealFunction : IHardMultivariateRealFunction
    {
        /// <summary>
        /// The pointer to the method, which represents the function.
        /// </summary>
        private HardMultivariateRealFunctionPointer hardRealFunctionPointer;

        /// <summary>
        /// Initializes a new instance of the <see cref="HardMultivariateRealFunction"/> class.
        /// </summary>
        /// <param name="hardRealFunctionPointer">The pointer to the method, which represents the function.</param>
        public HardMultivariateRealFunction(HardMultivariateRealFunctionPointer hardRealFunctionPointer)
        {
            if (hardRealFunctionPointer == (HardMultivariateRealFunctionPointer) null)
            {
                throw new ArgumentNullException("hardRealFunctionPointer");
            }

            this.hardRealFunctionPointer = hardRealFunctionPointer;
        }

        /// <summary>
        /// Gets or sets the pointer to the method, which represents the function.
        /// </summary>
        /// <value>The pointer to the method, which represents the function.</value>
        public HardMultivariateRealFunctionPointer HardRealFunctionPointer
        {
            get { return hardRealFunctionPointer; }
            set { hardRealFunctionPointer = value; }
        }

        /// <summary>
        /// Calculates the function at the specified vector.
        /// </summary>
        /// <param name="vector">The input values for the function in form of a vector.</param>
        /// <returns>
        /// The result of the function at the specified vector.
        /// </returns>
        public double SolveAt(GeneralVector vector)
        {
            return this.hardRealFunctionPointer(vector);
        }
    }
}