﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// Defines a function delegate to bind the method, which represents the function.
    /// </summary>
    public delegate double HardRealFunctionPointer(double x);

    /// <summary>
    /// This class represents a function, which can be implemented by source code.
    /// </summary>
    [Serializable]
    public class HardRealFunction : IRealFunction
    {
        /// <summary>
        /// The pointer to the method, which represents the function.
        /// </summary>
        private HardRealFunctionPointer hardRealFunctionPointer;

        /// <summary>
        /// Initializes a new instance of the <see cref="HardRealFunction"/> class.
        /// </summary>
        /// <param name="hardRealFunctionPointer">The pointer to the method, which represents the function.</param>
        public HardRealFunction(HardRealFunctionPointer hardRealFunctionPointer)
        {
            this.hardRealFunctionPointer = hardRealFunctionPointer;
        }

        /// <summary>
        /// Gets or sets the pointer to the method, which represents the function.
        /// </summary>
        /// <value>The pointer to the method, which represents the function.</value>
        public HardRealFunctionPointer HardRealFunctionPointer
        {
            get { return hardRealFunctionPointer; }
            set { hardRealFunctionPointer = value; }
        }

        /// <summary>
        /// Calculates the function at the specified value x.
        /// </summary>
        /// <param name="x">The specified value x.</param>
        /// <returns>The result of the function at the specified value x.</returns>
        public double SolveAt(double x)
        {
            return this.hardRealFunctionPointer(x);
        }
    }
}