﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// Defines a function delegate to bind the method, which represents the trivariate complex 
    /// function with three complex arguments.
    /// </summary>
    public delegate ComplexNumber HardTrivariateComplexFunctionPointer(ComplexNumber a, ComplexNumber b, ComplexNumber c
        );

    /// <summary>
    /// This class represents a trivariate function, which can be implemented by source code.
    /// </summary>
    [Serializable]
    public class HardTrivariateComplexFunction : IHardTrivariateComplexFunction
    {
        /// <summary>
        /// The pointer to the method, which represents the function.
        /// </summary>
        private HardTrivariateComplexFunctionPointer hardComplexFunctionPointer;

        /// <summary>
        /// Initializes a new instance of the <see cref="HardTrivariateComplexFunction"/> class.
        /// </summary>
        /// <param name="hardComplexFunctionPointer">The pointer to the method, which represents the function.</param>
        public HardTrivariateComplexFunction(HardTrivariateComplexFunctionPointer hardComplexFunctionPointer)
        {
            this.hardComplexFunctionPointer = hardComplexFunctionPointer;
        }

        /// <summary>
        /// Gets or sets the pointer to the method, which represents the function.
        /// </summary>
        /// <value>The pointer to the method, which represents the function.</value>
        public HardTrivariateComplexFunctionPointer HardComplexFunctionPointer
        {
            get { return hardComplexFunctionPointer; }
            set { hardComplexFunctionPointer = value; }
        }

        /// <summary>
        /// Calculates the function at the specified values a, b and c.
        /// </summary>
        /// <param name="a">The first parameter of the function.</param>
        /// <param name="b">The second parameter of the function.</param>
        /// <param name="c">The third parameter of the function.</param>
        /// <returns>
        /// The result of the function at the specified values a, b and c.
        /// </returns>
        public ComplexNumber SolveAt(ComplexNumber a, ComplexNumber b, ComplexNumber c)
        {
            return this.hardComplexFunctionPointer(a, b, c);
        }
    }
}