﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// Defines a function delegate to bind the method, which represents the trivariate real 
    /// function with three real arguments.
    /// </summary>
    public delegate double HardTrivariateRealFunctionPointer(double a, double b, double c);

    /// <summary>
    /// This class represents a trivariate function, which can be implemented by source code.
    /// </summary>
    [Serializable]
    public class HardTrivariateRealFunction : IHardTrivariateRealFunction
    {
        /// <summary>
        /// The pointer to the method, which represents the function.
        /// </summary>
        private HardTrivariateRealFunctionPointer hardRealFunctionPointer;

        /// <summary>
        /// Initializes a new instance of the <see cref="HardTrivariateRealFunction"/> class.
        /// </summary>
        /// <param name="hardRealFunctionPointer">The pointer to the method, which represents the function.</param>
        public HardTrivariateRealFunction(HardTrivariateRealFunctionPointer hardRealFunctionPointer)
        {
            this.hardRealFunctionPointer = hardRealFunctionPointer;
        }

        /// <summary>
        /// Gets or sets the pointer to the method, which represents the function.
        /// </summary>
        /// <value>The pointer to the method, which represents the function.</value>
        public HardTrivariateRealFunctionPointer HardRealFunctionPointer
        {
            get { return hardRealFunctionPointer; }
            set { hardRealFunctionPointer = value; }
        }

        /// <summary>
        /// Calculates the function at the specified values a, b and c.
        /// </summary>
        /// <param name="a">The first parameter of the function.</param>
        /// <param name="b">The second parameter of the function.</param>
        /// <param name="c">The third parameter of the function.</param>
        /// <returns>
        /// The result of the function at the specified values a, b and c.
        /// </returns>
        public double SolveAt(double a, double b, double c)
        {
            return this.hardRealFunctionPointer(a, b, c);
        }
    }
}