namespace SmartMathLibrary
{
    /// <summary>
    /// Interface to implement a ComplexLagrangePolynomial class.
    /// </summary>
    public interface IComplexLagrangePolynomial
    {
        /// <summary>
        /// Gets the x values of the polynomial.
        /// </summary>
        /// <value>The x values of the polynomial.</value>
        ComplexNumber[] XValues { get; }

        /// <summary>
        /// Gets the y values of the polynomial.
        /// </summary>
        /// <value>The y values of the polynomial.</value>
        ComplexNumber[] YValues { get; }

        /// <summary>
        /// Compares the current instance to the specified polynomial.
        /// </summary>
        /// <param name="compare">The polynomial to compare.</param>
        /// <returns>True if both polynomials are even otherwise, false.</returns>
        bool CompareTo(ComplexLagrangePolynomial compare);

        /// <summary>
        /// Solves the polynomial at the specified position x.
        /// </summary>
        /// <param name="x">The value for evaluation.</param>
        /// <returns>The result f(x) at the specified position x.</returns>
        ComplexNumber SolveAt(ComplexNumber x);

        /// <summary>
        /// Converts the Lagrange polynomial to a normal polynomial.
        /// </summary>
        /// <returns>The converted polynomial.</returns>
        ComplexPolynomial ToPolynomial();
    }
}