namespace SmartMathLibrary
{
    /// <summary>
    /// Interface to implement a CubicSplineComponent class.
    /// </summary>
    public interface ICubicSplineComponent
    {
        /// <summary>
        /// Gets or sets the polynomial of the splinepart.
        /// </summary>
        /// <value>The polynomial of the splinepart.</value>
        SimplePolynomial Polynomial { get; set; }

        /// <summary>
        /// Gets or sets the start point of the splinepart.
        /// </summary>
        /// <value>The startpoint of the splinepart.</value>
        Point2D Start { get; set; }

        /// <summary>
        /// Gets or sets the endpoint of the splinepart.
        /// </summary>
        /// <value>The endpoint of the splinepart.</value>
        Point2D End { get; set; }

        /// <summary>
        /// Copies the current instance of a CubicSplineComponent.
        /// </summary>
        /// <returns>A copy of the current instance.</returns>
        CubicSplineComponent Copy();

        /// <summary>
        /// Determines whether [the current CubicSplineComponent is defined for] [the specified x value].
        /// </summary>
        /// <param name="x">The x value to check.</param>
        /// <returns>
        /// 	<c>true</c> if [the current CubicSplineComponent is defined for] [the specified x value]; otherwise, <c>false</c>.
        /// </returns>
        bool IsDefinedFor(double x);

        /// <summary>
        /// Compares the current instance to another CubicSplineComponent.
        /// </summary>
        /// <param name="compare">The CubicSplineComponent to compare.</param>
        /// <returns>True if the components are even otherwise, false.</returns>
        bool CompareTo(CubicSplineComponent compare);
    }
}