namespace SmartMathLibrary
{
    /// <summary>
    /// Interface to implement a LagrangePolynomial class.
    /// </summary>
    public interface ILagrangePolynomial
    {
        /// <summary>
        /// Gets the x values of the polynomial.
        /// </summary>
        /// <value>The x values of the polynomial.</value>
        double[] XValues { get; }

        /// <summary>
        /// Gets the y values of the polynomial.
        /// </summary>
        /// <value>The y values of the polynomial.</value>
        double[] YValues { get; }

        /// <summary>
        /// Compares the current instance to the specified polynomial.
        /// </summary>
        /// <param name="compare">The polynomial to compare.</param>
        /// <returns>True if both polynomials are even otherwise, false.</returns>
        bool CompareTo(LagrangePolynomial compare);

        /// <summary>
        /// Solves the polynomial at the specified position x.
        /// </summary>
        /// <param name="x">The value for evaluation.</param>
        /// <returns>The result f(x) at the specified position x.</returns>
        double SolveAt(double x);

        /// <summary>
        /// Converts the Lagrange polynomial to a normal polynomial.
        /// </summary>
        /// <returns>The converted polynomial.</returns>
        SimplePolynomial ToPolynomial();
    }
}