﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.IO
{
    /// <summary>
    /// This class provides methods to write one or more complex matrices to a xml file.
    /// </summary>
    [Serializable]
    public class ComplexMatrixFileWriter : AbstractXmlFileWriter
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexMatrixFileWriter"/> class.
        /// </summary>
        /// <param name="stream">The stream of the file writer.</param>
        /// <param name="encoding">The encoding of the file.</param>
        public ComplexMatrixFileWriter(Stream stream, Encoding encoding)
            : base(stream, encoding)
        {
            this.writeEngine.WriteStartElement("ComplexMatrices");
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexMatrixFileWriter"/> class.
        /// </summary>
        /// <param name="filename">The filename of the file to create.</param>
        /// <param name="encoding">The encoding of the file.</param>
        public ComplexMatrixFileWriter(String filename, Encoding encoding)
            : base(filename, encoding)
        {
            this.writeEngine.WriteStartElement("ComplexMatrices");
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexMatrixFileWriter"/> class.
        /// </summary>
        /// <param name="textWriter">The text writer of the writer.</param>
        public ComplexMatrixFileWriter(TextWriter textWriter)
            : base(textWriter)
        {
            this.writeEngine.WriteStartElement("ComplexMatrices");
        }

        /// <summary>
        /// Writes the specified complex matrix to the specified file.
        /// </summary>
        /// <param name="matrix">The complex matrix to write.</param>
        public void Write(ComplexMatrix matrix)
        {
            this.writeEngine.WriteStartElement("ComplexMatrix");

            for (int rows = 0; rows < matrix.Rows; rows++)
            {
                this.writeEngine.WriteStartElement("Row");

                for (int columns = 0; columns < matrix.Columns; columns++)
                {
                    this.writeEngine.WriteStartElement("Column");
                    this.writeEngine.WriteElementString("RealNumberPart", matrix.GetValueAtPosition
                                                                              (new MatrixPosition(rows, columns)).
                                                                              RealNumberPart.ToString());
                    this.writeEngine.WriteElementString("ImaginaryNumberPart", matrix.GetValueAtPosition
                                                                                   (new MatrixPosition(rows, columns)).
                                                                                   ImaginaryNumberPart.ToString());
                    this.writeEngine.WriteEndElement();
                }

                this.writeEngine.WriteEndElement();
            }

            this.writeEngine.WriteEndElement();
        }

        /// <summary>
        /// Closes the writer and saves the output values.
        /// </summary>
        public override void Close()
        {
            this.writeEngine.WriteEndElement();
            base.Close();
        }
    }
}