﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.IO
{
    /// <summary>
    /// This class provides methods to write one or more vectors to a xml file.
    /// </summary>
    [Serializable]
    public class GeneralVectorFileWriter : AbstractXmlFileWriter
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVectorFileWriter"/> class.
        /// </summary>
        /// <param name="stream">The stream of the file writer.</param>
        /// <param name="encoding">The encoding of the file.</param>
        public GeneralVectorFileWriter(Stream stream, Encoding encoding)
            : base(stream, encoding)
        {
            this.writeEngine.WriteStartElement("Vectors");
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVectorFileWriter"/> class.
        /// </summary>
        /// <param name="filename">The filename of the file to create.</param>
        /// <param name="encoding">The encoding of the file.</param>
        public GeneralVectorFileWriter(String filename, Encoding encoding)
            : base(filename, encoding)
        {
            this.writeEngine.WriteStartElement("Vectors");
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVectorFileWriter"/> class.
        /// </summary>
        /// <param name="textWriter">The text writer of the writer.</param>
        public GeneralVectorFileWriter(TextWriter textWriter)
            : base(textWriter)
        {
            this.writeEngine.WriteStartElement("Vectors");
        }

        /// <summary>
        /// Writes the specified vector to the specified file.
        /// </summary>
        /// <param name="vector">The vector to write.</param>
        public void Write(GeneralVector vector)
        {
            this.writeEngine.WriteStartElement("Vector");

            for (int i = 0; i < vector.Count; i++)
            {
                this.writeEngine.WriteElementString("Column", vector.GetComponent(i).ToString());
            }

            this.writeEngine.WriteEndElement();
        }

        /// <summary>
        /// Closes the writer and saves the output values.
        /// </summary>
        public override void Close()
        {
            this.writeEngine.WriteEndElement();
            base.Close();
        }
    }
}