﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.IO
{
    /// <summary>
    /// This class provides methods to write one or more polynomials to a xml file.
    /// </summary>
    [Serializable]
    public class PolynomialFileWriter : AbstractXmlFileWriter
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVectorFileWriter"/> class.
        /// </summary>
        /// <param name="stream">The stream of the file writer.</param>
        /// <param name="encoding">The encoding of the file.</param>
        public PolynomialFileWriter(Stream stream, Encoding encoding)
            : base(stream, encoding)
        {
            this.writeEngine.WriteStartElement("Polynomials");
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVectorFileWriter"/> class.
        /// </summary>
        /// <param name="filename">The filename of the file to create.</param>
        /// <param name="encoding">The encoding of the file.</param>
        public PolynomialFileWriter(String filename, Encoding encoding)
            : base(filename, encoding)
        {
            this.writeEngine.WriteStartElement("Polynomials");
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVectorFileWriter"/> class.
        /// </summary>
        /// <param name="textWriter">The text writer of the writer.</param>
        public PolynomialFileWriter(TextWriter textWriter)
            : base(textWriter)
        {
            this.writeEngine.WriteStartElement("Polynomials");
        }

        /// <summary>
        /// Writes the specified polynomial to the specified file.
        /// </summary>
        /// <param name="polynomial">The polynomial to write.</param>
        public void Write(Polynomial polynomial)
        {
            this.writeEngine.WriteStartElement("Polynomial");

            for (int i = 0; i < polynomial.Components.Count; i++)
            {
                this.writeEngine.WriteStartElement("Element");
                this.writeEngine.WriteElementString("Coefficient", polynomial.Components[i].Coefficient.ToString());
                this.writeEngine.WriteElementString("Exponent", polynomial.Components[i].Exponent.ToString());
                this.writeEngine.WriteEndElement();
            }

            this.writeEngine.WriteEndElement();
        }

        /// <summary>
        /// Closes the writer and saves the output values.
        /// </summary>
        public override void Close()
        {
            this.writeEngine.WriteEndElement();
            base.Close();
        }
    }
}