﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary;

namespace SmartMathLibrary.LinearEquationSolvers.Preconditioners
{
    /// <summary>
    /// This class represents a matrix preconditioner, which calculates the precondition matrix by calculation 
    /// of the inverse matrix.
    /// </summary>
    [Serializable]
    public class DirectInversePreconditioner : IPreconditioner
    {
        /// <summary>
        /// The source matrix for the precondition process.
        /// </summary>
        private SquareMatrix sourceMatrix;

        /// <summary>
        /// Initializes a new instance of the <see cref="DirectInversePreconditioner"/> class.
        /// </summary>
        /// <param name="sourceMatrix">The source matrix for the precondition process.</param>
        public DirectInversePreconditioner(SquareMatrix sourceMatrix)
        {
            this.sourceMatrix = sourceMatrix;
        }

        /// <summary>
        /// Gets or sets the source matrix for the precondition process.
        /// </summary>
        /// <value>The source matrix for the precondition process.</value>
        public SquareMatrix SourceMatrix
        {
            get { return sourceMatrix; }
            set { sourceMatrix = value; }
        }

        /// <summary>
        /// Computes the precondition matrix of the specified matrix.
        /// </summary>
        /// <returns>
        /// The computed precondition matrix of the original matrix.
        /// </returns>
        public Matrix ComputePreconditionMatrix()
        {
            return InverseMatrix.FromMatrix(this.sourceMatrix);
        }
    }
}