﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary;

namespace SmartMathLibrary.LinearEquationSolvers.Preconditioners
{
    /// <summary>
    /// This class represents a matrix preconditioner, which do nothing than returning the original matrix.
    /// </summary>
    [Serializable]
    public class NoPreconditioner : IPreconditioner
    {
        /// <summary>
        /// The source matrix for the precondition process.
        /// </summary>
        private Matrix sourceMatrix;

        /// <summary>
        /// Initializes a new instance of the <see cref="NoPreconditioner"/> class.
        /// </summary>
        /// <param name="sourceMatrix">The source matrix for the precondition process.</param>
        public NoPreconditioner(Matrix sourceMatrix)
        {
            if (sourceMatrix == (Matrix) null)
            {
                throw new ArgumentNullException("sourceMatrix");
            }

            this.sourceMatrix = sourceMatrix;
        }

        /// <summary>
        /// Gets or sets the source matrix for the precondition process.
        /// </summary>
        /// <value>The source matrix for the precondition process.</value>
        public Matrix SourceMatrix
        {
            get { return sourceMatrix; }
            set { sourceMatrix = value; }
        }

        /// <summary>
        /// Computes the precondition matrix of the specified matrix.
        /// </summary>
        /// <returns>
        /// The computed precondition matrix of the original matrix.
        /// </returns>
        public Matrix ComputePreconditionMatrix()
        {
            return this.sourceMatrix.Copy();
        }
    }
}