﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace SmartMathLibrary.Management.Performance
{
    /// <summary>
    /// This class provides a special thread, which allows the developer to suspend and resume the 
    /// thread sessions. Overwrite the Work(object) method to realize the thread action method.
    /// </summary>
    [Serializable]
    public abstract class AbstractSuspendableParameterizedThread : AbstractThread
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractSuspendableParameterizedThread"/> class.
        /// </summary>
        protected AbstractSuspendableParameterizedThread()
            : base()
        {
            this.thread = new Thread(new ParameterizedThreadStart(this.Worker));
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractSuspendableParameterizedThread"/> class.
        /// </summary>
        /// <param name="refreshRate">The refresh rate for checking if the thread was set to resume state.</param>
        protected AbstractSuspendableParameterizedThread(TimeSpan refreshRate)
            : base(refreshRate)
        {
            this.thread = new Thread(new ParameterizedThreadStart(this.Worker));
        }

        /// <summary>
        /// This method provides the thread start. Overwrite this method to use it.
        /// </summary>
        /// <param name="obj">The parameter, which may used in the thread.</param>
        public abstract void Worker(object obj);
    }
}