﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.NonlinearEquationSolvers
{
    /// <summary>
    /// This class represents an abstract layer for root finders, which need a derivation.
    /// </summary>
    [Serializable]
    public abstract class AbstractDerivativeNeedRootFinder : AbstractRootFinder
    {
        /// <summary>
        /// The polynomial for finding the roots.
        /// </summary>
        private Polynomial polynomial;

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractDerivativeNeedRootFinder"/> class.
        /// </summary>
        /// <param name="polynomial">The polynomial for finding the roots.</param>
        protected AbstractDerivativeNeedRootFinder(Polynomial polynomial)
        {
            if (polynomial == (Polynomial) null)
            {
                throw new ArgumentNullException("polynomial");
            }

            this.polynomial = polynomial.Copy();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractDerivativeNeedRootFinder"/> class.
        /// </summary>
        /// <param name="polynomial">The polynomial for finding the roots.</param>
        protected AbstractDerivativeNeedRootFinder(SimplePolynomial polynomial)
        {
            if (polynomial == (SimplePolynomial) null)
            {
                throw new ArgumentNullException("polynomial");
            }

            this.polynomial = polynomial.Copy().ToPolynomial();
        }

        /// <summary>
        /// Gets or sets the polynomial for finding the roots.
        /// </summary>
        /// <value>The polynomial for finding the roots.</value>
        public Polynomial Polynomial
        {
            get { return polynomial; }
            set { polynomial = value; }
        }
    }
}