﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.NumericalIntegration
{
    /// <summary>
    /// This class represents a complex integral.
    /// </summary>
    [Serializable]
    public class ComplexFinitelyIntegral : IComplexFinitelyIntegral
    {
        /// <summary>
        /// The lower value a of the integral.
        /// </summary>
        private ComplexNumber a;

        /// <summary>
        /// The upper value b of the integral.
        /// </summary>
        private ComplexNumber b;

        /// <summary>
        /// The function of the integral.
        /// </summary>
        private IComplexFunction function;

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexFinitelyIntegral"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="function">The function of the integral.</param>
        public ComplexFinitelyIntegral(ComplexNumber a, ComplexNumber b, IComplexFunction function)
        {
            this.a = a;
            this.b = b;
            this.function = function;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexFinitelyIntegral"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="function">The function of the integral.</param>
        public ComplexFinitelyIntegral(ComplexNumber a, ComplexNumber b, ComplexPolynomial function)
        {
            this.a = a;
            this.b = b;
            this.function = function;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexFinitelyIntegral"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="function">The function of the integral.</param>
        public ComplexFinitelyIntegral(ComplexNumber a, ComplexNumber b, HardComplexFunction function)
        {
            this.a = a;
            this.b = b;
            this.function = function;
        }

        /// <summary>
        /// Gets or sets the lower value a of the integral.
        /// </summary>
        /// <value>The lower value a of the integral.</value>
        public ComplexNumber A
        {
            get { return a; }
            set { a = value; }
        }

        /// <summary>
        /// Gets or sets the upper value b of the integral.
        /// </summary>
        /// <value>The upper value b of the integral.</value>
        public ComplexNumber B
        {
            get { return b; }
            set { b = value; }
        }

        /// <summary>
        /// Gets or sets the function of the integral.
        /// </summary>
        /// <value>The function of the integral.</value>
        public IComplexFunction Function
        {
            get { return function; }
            set { function = value; }
        }
    }
}