﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary;

namespace SmartMathLibrary.NumericalIntegration
{
    /// <summary>
    /// This class represents a finity linear spline integral.
    /// </summary>
    [Serializable]
    public class FinityLinearSplineIntegral : IFinityLinearSplineIntegral
    {
        /// <summary>
        /// The lower value a of the integral.
        /// </summary>
        private double a;

        /// <summary>
        /// The upper value b of the integral.
        /// </summary>
        private double b;

        /// <summary>
        /// The function of the integral.
        /// </summary>
        private LinearSpline function;

        /// <summary>
        /// Initializes a new instance of the <see cref="FinityLinearSplineIntegral"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="function">The function of the integral.</param>
        public FinityLinearSplineIntegral(double a, double b, LinearSpline function)
        {
            this.a = a;
            this.b = b;
            this.function = function;
        }

        /// <summary>
        /// Gets or sets the lower value a of the integral.
        /// </summary>
        /// <value>The lower value a of the integral.</value>
        public double A
        {
            get { return a; }
            set { a = value; }
        }

        /// <summary>
        /// Gets or sets the upper value b of the integral.
        /// </summary>
        /// <value>The upper value b of the integral.</value>
        public double B
        {
            get { return b; }
            set { b = value; }
        }

        /// <summary>
        /// Gets or sets the function of the integral.
        /// </summary>
        /// <value>The function of the integral.</value>
        public LinearSpline Function
        {
            get { return function; }
            set { function = value; }
        }
    }
}