﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class provides a scale matrix for three dimensional vectors.
    /// </summary>
    [Serializable]
    public class ScaleMatrix3D
    {
        /// <summary>
        /// The scale factor for the x axis.
        /// </summary>
        private readonly double scaleFactorX;

        /// <summary>
        /// The scale factor for the y axis.
        /// </summary>
        private readonly double scaleFactorY;

        /// <summary>
        /// The scale factor for the z axis.
        /// </summary>
        private readonly double scaleFactorZ;

        /// <summary>
        /// Initializes a new instance of the <see cref="ScaleMatrix3D"/> class.
        /// </summary>
        /// <param name="scaleFactor">The scale factor for the x, y and z axis.</param>
        public ScaleMatrix3D(double scaleFactor)
        {
            this.scaleFactorX = scaleFactor;
            this.scaleFactorY = scaleFactor;
            this.scaleFactorZ = scaleFactor;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ScaleMatrix3D"/> class.
        /// </summary>
        /// <param name="scaleFactorX">The scale factor for the x axis.</param>
        /// <param name="scaleFactorY">The scale factor for the y axis.</param>
        /// <param name="scaleFactorZ">The scale factor for the z axis.</param>
        public ScaleMatrix3D(double scaleFactorX, double scaleFactorY, double scaleFactorZ)
        {
            this.scaleFactorX = scaleFactorX;
            this.scaleFactorY = scaleFactorY;
            this.scaleFactorZ = scaleFactorZ;
        }

        /// <summary>
        /// Gets the scale matrix for the transformation.
        /// </summary>
        /// <returns>The scale matrix for the transformation.</returns>
        public Matrix GetScaleMatrix()
        {
            double[,] matrixData = new double[3,3];

            matrixData[0, 0] = this.scaleFactorX;
            matrixData[0, 1] = 0;
            matrixData[0, 2] = 0;

            matrixData[1, 0] = 0;
            matrixData[1, 1] = this.scaleFactorY;
            matrixData[1, 2] = 0;

            matrixData[2, 0] = 0;
            matrixData[2, 1] = 0;
            matrixData[2, 2] = this.scaleFactorZ;

            return new Matrix(matrixData);
        }
    }
}