﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Space.CoreTypes
{
    /// <summary>
    /// This class represents a numeric integer value.
    /// </summary>
    [Serializable]
    public class SpaceInteger : ISpaceEntry
    {
        /// <summary>
        /// The specified numeric integer value.
        /// </summary>
        [SpaceAttribute] 
        private int data;

        /// <summary>
        /// Initializes a new instance of the <see cref="SpaceInteger"/> class.
        /// </summary>
        /// <param name="data">The specified numeric integer value.</param>
        public SpaceInteger(int data)
        {
            this.data = data;
        }

        /// <summary>
        /// Gets or sets the specified numeric integer value.
        /// </summary>
        /// <value>The specified numeric integer value.</value>
        public int Data
        {
            get { return this.data; }
            set { this.data = value; }
        }

        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        /// A new object that is a copy of this instance.
        /// </returns>
        public object Clone()
        {
            return CloneFunctions.Clone(this);
        }

        /// <summary>
        /// Compares the current <see cref="ISpaceEntry"/> to the specified <see cref="ISpaceEntry"/>.
        /// </summary>
        /// <param name="entry">The specified entry to compare.</param>
        /// <returns>
        /// If the object instances are even, the method returns true otherwise, false.
        /// </returns>
        public bool Equals(ISpaceEntry entry)
        {
            if (entry is SpaceInteger)
            {
                SpaceInteger check = (SpaceInteger)entry;

                return check.Data == this.Data;
            }

            return false;
        }

        /// <summary>
        /// Compares the current <see cref="ISpaceEntry"/> to the specified <see cref="ISpaceEntry"/>.
        /// </summary>
        /// <param name="entry">The specified entry to compare.</param>
        /// <returns>
        /// If the specified object instance is lower than the current instance the method returns true otherwise,
        /// false.
        /// </returns>
        public bool LowerThan(ISpaceEntry entry)
        {
            if (entry is SpaceInteger)
            {
                SpaceInteger check = (SpaceInteger)entry;

                return this.Data < check.Data;
            }

            return false;
        }

        /// <summary>
        /// Compares the current <see cref="ISpaceEntry"/> to the specified <see cref="ISpaceEntry"/>.
        /// </summary>
        /// <param name="entry">The specified entry to compare.</param>
        /// <returns>
        /// If the specified object instance is greater than the current instance the method returns true otherwise,
        /// false.
        /// </returns>
        public bool GreaterThan(ISpaceEntry entry)
        {
            if (entry is SpaceInteger)
            {
                SpaceInteger check = (SpaceInteger)entry;

                return this.Data > check.Data;
            }

            return false;
        }
    }
}