﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary.Space.CoreTypes;

namespace SmartMathLibrary.Space.QueryEngine
{
    /// <summary>
    /// This class represents an integral function query, which allows the extraction of all space integrals 
    /// with a specified function.
    /// </summary>
    [Serializable]
    public class IntegralFunctionQuery : AbstractSpaceQuery
    {
        /// <summary>
        /// The compare value.
        /// </summary>
        private Polynomial value;

        /// <summary>
        /// Initializes a new instance of the <see cref="IntegralFunctionQuery"/> class.
        /// </summary>
        /// <param name="value">The compare value.</param>
        public IntegralFunctionQuery(Polynomial value)
            : base()
        {
            this.value = value;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="IntegralFunctionQuery"/> class.
        /// </summary>
        /// <param name="value">The compare value.</param>
        /// <param name="status">The object status criteria.</param>
        public IntegralFunctionQuery(Polynomial value, SpaceObjectStatus status)
            : base(status)
        {
            this.value = value;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="IntegralFunctionQuery"/> class.
        /// </summary>
        /// <param name="value">The compare value.</param>
        /// <param name="limit">The maximum number of query results.</param>
        public IntegralFunctionQuery(Polynomial value, int limit)
            : base(limit)
        {
            this.value = value;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="IntegralFunctionQuery"/> class.
        /// </summary>
        /// <param name="value">The compare value.</param>
        /// <param name="limit">The maximum number of query results.</param>
        /// <param name="status">The object status criteria.</param>
        public IntegralFunctionQuery(Polynomial value, int limit, SpaceObjectStatus status)
            : base(limit, status)
        {
            this.value = value;
        }

        /// <summary>
        /// Gets or sets the compare value.
        /// </summary>
        /// <value>The compare value</value>
        public Polynomial Value
        {
            get { return this.value; }
            set { this.value = value; }
        }

        /// <summary>
        /// Defines the query criteria of the specified query.
        /// </summary>
        /// <param name="item">The query source items.</param>
        /// <returns>
        /// The space ID fulfills the query criteria the method returns true otherwise, false.
        /// </returns>
        public override bool QueryDefinition(ISpaceEntry item)
        {
            if (item is SpaceIntegral)
            {
                return ((SpaceIntegral) item).Function == this.value;
            }

            return false;
        }
    }
}