﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class supports the development of Taylor series out of polynomials.
    /// </summary>
    [Serializable]
    public class TaylorSeriesDevelopment : ITaylorSeriesDevelopment
    {
        /// <summary>
        /// The base polynomial for developing the Taylor series.
        /// </summary>
        private readonly Polynomial basePolynomial;

        /// <summary>
        /// Initializes a new instance of the <see cref="TaylorSeriesDevelopment"/> class.
        /// </summary>
        /// <param name="polynomial">The base polynomial for developing the Taylor series.</param>
        public TaylorSeriesDevelopment(Polynomial polynomial)
        {
            if (polynomial == (Polynomial) null)
            {
                throw new ArgumentNullException("polynomial");
            }

            this.basePolynomial = polynomial;
        }

        /// <summary>
        /// Develops the Taylor series.
        /// </summary>
        /// <param name="a">The function about point of the series.</param>
        /// <param name="n">The maximum derivative, which should be used to develop the series.</param>
        /// <returns>The developed Taylor series.</returns>
        public TaylorSeries DevelopSeries(double a, int n)
        {
            TaylorSeries result = new TaylorSeries();
            Polynomial tempuri = this.basePolynomial.Copy();

            result.ConstantValue = this.basePolynomial.SolveAt(a);

            for (int i = 1; i <= n; i++)
            {
                tempuri = tempuri.Derivative();

                double coefficient = tempuri.SolveAt(a) / Factorials.Factorial((uint) i);

                result.Components.Add(new TaylorSeriesComponent(coefficient, i, a));
            }

            return result;
        }
    }
}