﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class provides a translation matrix for three dimensional vectors. If a two dimensional
    /// vector should be transformed use for the three component of the vector the value one.
    /// </summary>
    [Serializable]
    public class TranslationMatrix3D
    {
        /// <summary>
        /// The x translation of the translation matrix.
        /// </summary>
        private double xTranslation;

        /// <summary>
        /// The y translation of the translation matrix.
        /// </summary>
        private double yTranslation;

        /// <summary>
        /// Initializes a new instance of the <see cref="TranslationMatrix3D"/> class.
        /// </summary>
        /// <param name="translation">The translation value for x, y and z axis.</param>
        public TranslationMatrix3D(double translation)
        {
            this.xTranslation = translation;
            this.yTranslation = translation;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TranslationMatrix3D"/> class.
        /// </summary>
        /// <param name="xTranslation">The translation value for the x axis.</param>
        /// <param name="yTranslation">The translation value for the y axis.</param>
        public TranslationMatrix3D(double xTranslation, double yTranslation)
        {
            this.xTranslation = xTranslation;
            this.yTranslation = yTranslation;
        }

        /// <summary>
        /// Gets or sets the translation value for the x axis.
        /// </summary>
        /// <value>The translation value for the x axis.</value>
        public double XTranslation
        {
            get { return xTranslation; }
            set { xTranslation = value; }
        }

        /// <summary>
        /// Gets or sets the translation value for the y axis.
        /// </summary>
        /// <value>The translation value for the y axis.</value>
        public double YTranslation
        {
            get { return yTranslation; }
            set { yTranslation = value; }
        }

        /// <summary>
        /// Gets the translation matrix for the transformation.
        /// </summary>
        /// <returns>The translation matrix for the transformation.</returns>
        public Matrix GetTranslationMatrix()
        {
            double[,] matrixData = new double[4,4];

            matrixData[0, 0] = 1;
            matrixData[0, 1] = 0;
            matrixData[0, 2] = this.xTranslation;

            matrixData[1, 0] = 0;
            matrixData[1, 1] = 1;
            matrixData[1, 2] = this.yTranslation;

            matrixData[2, 0] = 0;
            matrixData[2, 1] = 0;
            matrixData[2, 2] = 1;

            return new Matrix(matrixData);
        }
    }
}